/*
 * Copyright (c) 2018-2020 C4
 *
 * This file is part of Curios, a mod made for Minecraft.
 *
 * Curios is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Curios is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Curios.  If not, see <https://www.gnu.org/licenses/>.
 */

package top.theillusivec4.curios.common;

import com.google.common.collect.HashMultimap;
import com.google.common.collect.Multimap;
import nerdhub.cardinal.components.api.event.ItemComponentCallbackV2;
import net.fabricmc.fabric.api.screenhandler.v1.ScreenHandlerRegistry;
import net.minecraft.class_1293;
import net.minecraft.class_1294;
import net.minecraft.class_1309;
import net.minecraft.class_1320;
import net.minecraft.class_1322;
import net.minecraft.class_1322.class_1323;
import net.minecraft.class_1792;
import net.minecraft.class_2378;
import net.minecraft.class_2960;
import net.minecraft.class_3417;
import net.minecraft.class_3419;
import net.minecraft.class_3917;
import net.minecraft.class_5134;
import top.theillusivec4.curios.api.CuriosApi;
import top.theillusivec4.curios.api.CuriosComponent;
import top.theillusivec4.curios.api.type.component.ICurio;
import top.theillusivec4.curios.common.inventory.screen.CuriosScreenHandler;
import top.theillusivec4.curios.common.item.AmuletItem;
import top.theillusivec4.curios.common.item.CrownItem;
import top.theillusivec4.curios.common.item.KnucklesItem;
import top.theillusivec4.curios.common.item.RingItem;

public class CuriosRegistry {

  public static final class_1792 AMULET = new AmuletItem();
  public static final class_1792 CROWN = new CrownItem();
  public static final class_1792 RING = new RingItem();
  public static final class_1792 KNUCKLES = new KnucklesItem();

  public static final class_3917<CuriosScreenHandler> CURIOS_SCREENHANDLER = ScreenHandlerRegistry
      .registerSimple(new class_2960(CuriosApi.MODID, "curios_screen"), CuriosScreenHandler::new);

  public static void registerItems() {
    class_2378.method_10230(class_2378.field_11142, new class_2960(CuriosApi.MODID, "amulet"), AMULET);
    class_2378.method_10230(class_2378.field_11142, new class_2960(CuriosApi.MODID, "crown"), CROWN);
    class_2378.method_10230(class_2378.field_11142, new class_2960(CuriosApi.MODID, "ring"), RING);
    class_2378.method_10230(class_2378.field_11142, new class_2960(CuriosApi.MODID, "knuckles"), KNUCKLES);
  }

  public static void registerComponents() {
    ItemComponentCallbackV2.event(AMULET).register(
        ((item, itemStack, componentContainer) -> componentContainer
            .put(CuriosComponent.ITEM, new ICurio() {

              @Override
              public void curioTick(String identifier, int index, class_1309 livingEntity) {

                if (!livingEntity.method_5770().method_8608() && livingEntity.field_6012 % 40 == 0) {
                  livingEntity.method_6092(
                      new class_1293(class_1294.field_5924, 80, 0, true, true));
                }
              }
            })));

    ItemComponentCallbackV2.event(CROWN).register(
        ((item, itemStack, componentContainer) -> componentContainer
            .put(CuriosComponent.ITEM, new ICurio() {

              @Override
              public void curioTick(String identifier, int index, class_1309 livingEntity) {

                if (!livingEntity.method_5770().method_8608() && livingEntity.field_6012 % 20 == 0) {
                  livingEntity.method_6092(
                      new class_1293(class_1294.field_5925, 300, -1, true, true));
                  itemStack.method_7956(1, livingEntity, damager -> CuriosApi.getCuriosHelper()
                      .onBrokenCurio(identifier, index, damager));
                }
              }
            })));

    ItemComponentCallbackV2.event(RING).register(
        ((item, itemStack, componentContainer) -> componentContainer
            .put(CuriosComponent.ITEM, new ICurio() {
              @Override
              public void curioTick(String identifier, int index, class_1309 livingEntity) {

                if (!livingEntity.method_5770().method_8608() && livingEntity.field_6012 % 19 == 0) {
                  livingEntity.method_6092(
                      new class_1293(class_1294.field_5917, 20, 0, true, true));
                }
              }

              @Override
              public Multimap<class_1320, class_1322> getAttributeModifiers(
                  String identifier) {
                Multimap<class_1320, class_1322> attributes = HashMultimap
                    .create();

                if (CuriosApi.getCuriosHelper().getCurioTags(itemStack.method_7909())
                    .contains(identifier)) {
                  attributes.put(class_5134.field_23719,
                      new class_1322(RingItem.SPEED_UUID, "Speed bonus", 0.1,
                          class_1323.field_6331));
                  attributes.put(class_5134.field_23724,
                      new class_1322(RingItem.ARMOR_UUID, "Armor bonus", 2,
                          class_1323.field_6328));
                }
                return attributes;
              }

              @Override
              public void playRightClickEquipSound(class_1309 livingEntity) {
                livingEntity.field_6002
                    .method_8396(null, livingEntity.method_24515(), class_3417.field_14761,
                        class_3419.field_15254, 1.0F, 1.0F);
              }

              @Override
              public boolean canRightClickEquip() {
                return true;
              }

              @Override
              public DropRule getDropRule(class_1309 livingEntity) {
                return DropRule.ALWAYS_KEEP;
              }
            })));

    ItemComponentCallbackV2.event(KNUCKLES).register(
        ((item, itemStack, componentContainer) -> componentContainer
            .put(CuriosComponent.ITEM, new ICurio() {

              @Override
              public Multimap<class_1320, class_1322> getAttributeModifiers(
                  String identifier) {
                Multimap<class_1320, class_1322> atts = HashMultimap.create();

                if (CuriosApi.getCuriosHelper().getCurioTags(itemStack.method_7909())
                    .contains(identifier)) {
                  atts.put(class_5134.field_23721,
                      new class_1322(KnucklesItem.ATTACK_DAMAGE_UUID,
                          "Attack damage bonus", 4, class_1323.field_6328));
                }
                return atts;
              }
            })));
  }
}
